#include <iostream>
#include <iomanip>
#include <functional>
#include <string>
#include <unordered_set>
#include <boost/functional/hash.hpp>
using namespace std;

struct S {
	string first_name;
	string last_name;
};
bool operator==(const S& lhs, const S& rhs) {
	return lhs.first_name == rhs.first_name && lhs.last_name == rhs.last_name;
}

// custom hash can be a standalone function object:
struct MyHash {
	size_t operator()(S const& s) const noexcept {
//		size_t h1 = hash<string>{}(s.first_name);
//		size_t h2 = hash<string>{}(s.last_name);
//		return h1 ^ (h2 << 1); // or use boost::hash_combine
		std::size_t seed = 0;
		boost::hash_combine(seed, s.first_name);
		boost::hash_combine(seed, s.last_name);
		return seed;
	}
};

// custom specialization of hash can be injected in namespace std
template<>
struct std::hash<S> {
	size_t operator()(S const& s) const noexcept {
//		size_t h1 = hash<string>{}(s.first_name);
//		size_t h2 = hash<string>{}(s.last_name);
//		return h1 ^ (h2 << 1); // or use boost::hash_combine
		std::size_t seed = 0;
		boost::hash_combine(seed, s.first_name);
		boost::hash_combine(seed, s.last_name);
		return seed;
	}
};

int main() {
	string str = "Meet the new boss...";
	size_t str_hash = hash<string>{}(str);
	cout << "hash(" << quoted(str) << ") = " << str_hash << '\n';

	S obj = { "Hubert", "Farnsworth" };
	// using the standalone function object
	cout << "hash(" << quoted(obj.first_name) << ", "
	          << quoted(obj.last_name) << ") = "
	          << MyHash{}(obj) << " (using MyHash)\n" << setw(31) << "or "
	          << hash<S>{}(obj) << " (using injected hash<S> specialization)\n";

	// custom hash makes it possible to use custom types in unordered containers
	// The example will use the injected hash<S> specialization above,
	// to use MyHash instead, pass it as a second template argument
	unordered_set<S> names = {
		obj, {"Bender", "John"}, {"Turing", "Jack"} };
	for(auto& s: names)
		cout << quoted(s.first_name) << ' ' << quoted(s.last_name) << '\n';
}

