#include <vector>
#include <iostream>
using namespace std;

class Noisy {
	static long create, assign, copy_ctor, destroy, move_ctor;
	long id;
public:
	Noisy() : id(create++) { 
		cout << "d[" << id << "]"; 
	}
	Noisy(const Noisy& noisy) : id(noisy.id) {
		cout << "c[" << id << "]";
		copy_ctor++;
	}
    Noisy(Noisy&& noisy)  noexcept : id(noisy.id) {
        cout << "m[" << id << "]";
        move_ctor++;
    }
	Noisy& operator=(const Noisy& noisy) {
		cout << "(" << id << ")=[" << noisy.id << "]";
		id = noisy.id;
		assign++;
		return *this;
	}
	friend bool operator<(const Noisy& lv, const Noisy& rv) {
		return lv.id < rv.id;
	}
	friend bool operator==(const Noisy& lv, const Noisy& rv) {
		return lv.id == rv.id;
	}
	~Noisy() {
		cout << "~[" << id << "]";
		destroy++;
	}
	friend ostream& operator<<(ostream& os, const Noisy& noisy) {
		return os << noisy.id;
	}
	friend class NoisyReport;
};

class NoisyReport {
	static NoisyReport nr;	//	a hack to create NoisyReport static object
public:
	~NoisyReport() {
		cout << endl;
		cout << endl;
		cout << "cleaning up" << endl;
		cout << "-------------------" << endl;
		cout << "Noisy creations: " << Noisy::create << endl;
		cout << "Copy-Constructions: " << Noisy::copy_ctor << endl;
		cout << "Move-Constructions: " << Noisy::move_ctor << endl;
		cout <<	"Assignments: " << Noisy::assign << endl;
		cout << "Destructions: " << Noisy::destroy << endl;
		cout << endl;
	}
};

long Noisy::create = 0, Noisy::assign = 0, Noisy::move_ctor = 0;
long Noisy::copy_ctor = 0, Noisy::destroy = 0;
NoisyReport NoisyReport::nr;

int main(int argc, char* argv[]) {
	int size = 10;
	if(argc >= 2) size = atoi(argv[1]);
	vector<Noisy> vn;
	for(int i = 0; i < size; i++) {
		cout << "inserting element " << i << " ";
		vn.emplace_back(Noisy());
		cout << " : " << vn.size() << " " << vn.capacity() << endl;
	}
	cout << "------------------------------------" << endl;
	return 0;
}

